## START: Set by rpmautospec
## (rpmautospec version 0.2.6)
%define autorelease(e:s:pb:) %{?-p:0.}%{lua:
    release_number = 8;
    base_release_number = tonumber(rpm.expand("%{?-b*}%{!?-b:1}"));
    print(release_number + base_release_number - 1);
}%{?-e:.%{-e*}}%{?-s:.%{-s*}}%{?dist}
## END: Set by rpmautospec

%global _description %{expand:
Arbor is a high-performance library for Computational Neuroscience simulations.

Some key features include:

- Asynchronous spike exchange that overlaps compute and communication.
- Efficient sampling of voltage and current on all back ends.
- Efficient implementation of all features on GPU.
- Reporting of memory and energy consumption (when available on platform).
- An API for addition of new cell types, e.g. LIF and Poisson spike generators.
- Validation tests against numeric/analytic models and NEURON.

Documentation is available at https://arbor.readthedocs.io/en/latest/
}

# Best to start with the serial version when debugging build failures
%bcond_without mpich
%bcond_without openmpi

%bcond_without tests

%global forgeurl            https://github.com/arbor-sim/arbor

Name:           arbor
Version:        0.5.2
Release:        %autorelease
Summary:        Multi-compartment neural network simulation library

%forgemeta

URL:            %forgeurl
Source0:        %forgesource
# script to run examples
Source1:        https://raw.githubusercontent.com/arbor-sim/arbor/master/scripts/run_python_examples.sh
Source2:        https://raw.githubusercontent.com/arbor-sim/arbor/master/scripts/run_cpp_examples.sh
License:        BSD

# This patch changes ext/CMakeLists.txt for automatically using tinyopt libraries by cmake command.
#Patch1:         arbor-tinyopt_cmake.patch
# https://github.com/arbor-sim/arbor/pull/1899
Patch2:         arbor-remove-checks-for-move-ctor-pr1899.patch
# Extracted from: https://github.com/arbor-sim/arbor/pull/1625
# panda requires unique index
# https://github.com/arbor-sim/arbor/pull/1625/commits/eff34806dbea822798a5fd7a9e7600af29d126ab
Patch3:         arbor-pandas-prevent-duplicate-index-pr1625.patch

# Random123 does not support these
ExcludeArch:    mips64r2 mips32r2 s390

BuildRequires:  cmake
BuildRequires:  make
BuildRequires:  gcc-c++
BuildRequires:  git-core
BuildRequires:  gtest-devel
BuildRequires:  google-benchmark-devel
BuildRequires:  json-devel
BuildRequires:  libunwind-devel
BuildRequires:  pybind11-devel
BuildRequires:  python3-devel
BuildRequires:  python3-setuptools
BuildRequires:  Random123-devel
BuildRequires:  tclap-devel
Provides:       python3-arbor = %{version}-%{release}

# For validation, but we don't have these BRs
# BuildRequires:  julia julia-sundials julia-unitful julia-JSON

%if %{with tests}
BuildRequires:  %{py3_dist numpy}
BuildRequires:  %{py3_dist pandas}
BuildRequires:  %{py3_dist seaborn}
%endif

# Required by arbor
Requires:       %{py3_dist numpy}

%description %{_description}

%package devel
Summary:    Development files for arbor
Requires:   arbor%{?_isa} = %{version}-%{release}
Provides:   arbor-static = %{version}-%{release}

%description devel %{_description}

%package doc
# Does not require the main package, since it may be installed by people using
# the MPI builds
Summary:    Documentation for arbor
BuildRequires:  python3-sphinx
BuildRequires:  python3-sphinx_rtd_theme
BuildRequires:  python3-svgwrite

%description doc %{_description}

%if %{with mpich}
%package mpich
Summary:        MPICH build for arbor
BuildRequires:  mpich-devel
BuildRequires:  rpm-mpi-hooks
BuildRequires:  python3-mpi4py-mpich

Requires:       mpich
Requires:       python3-mpich
Requires:       python3-mpi4py-mpich
Requires:       %{py3_dist numpy}
Provides:       python3-arbor-mpich = %{version}-%{release}

%description mpich %{_description}

%package mpich-devel
Summary:    Development files for arbor-mpich
Requires:   arbor-mpich%{?_isa} = %{version}-%{release}
Provides:   arbor-mpich-static = %{version}-%{release}

%description mpich-devel %{_description}
%endif

%if %{with openmpi}
%package openmpi
Summary:        OpenMPI build for arbor
BuildRequires:  openmpi-devel
BuildRequires:  rpm-mpi-hooks
BuildRequires:  python3-mpi4py-openmpi

Requires:       openmpi
Requires:       python3-openmpi
Requires:       python3-mpi4py-openmpi
Requires:       %{py3_dist numpy}
Provides:       python3-arbor-openmpi = %{version}-%{release}

%description openmpi %{_description}

%package openmpi-devel
Summary:    Development files for arbor-openmpi
Requires:   arbor-openmpi%{?_isa} = %{version}-%{release}
Provides:   arbor-openmpi-static = %{version}-%{release}

%description openmpi-devel %{_description}
%endif

%prep
%forgesetup
%patch2 -p1 -b .move_ctor
%patch3 -p1 -b .pandas_index

# Disable failing padded test
sed -i '/test_padded.cpp/ d' test/unit/CMakeLists.txt

# should always be included, but is currently included only if bundled pybind11
# is used
sed -i '/add_subdirectory(python)/i \
include(FindPythonModule)' CMakeLists.txt
# Do not build external libraries
sed -i -e 's/ ext-random123//' CMakeLists.txt

# Disable doc build: we built it ourselves
sed -i '/add_subdirectory(doc)/ d' CMakeLists.txt

# Remove ext folders, unbundle libraries
rm -vrf ext/google-benchmark ext/json ext/random123
mv ext/tinyopt/LICENSE ext/tinyopt/LICENSE-tinyopt

# tclap and json are both header only
find . -type f -name "CMakeLists.txt" -exec sed -i -e 's/ext-tclap//' -e 's/ext-json//' {} 2>/dev/null ';'

# Correct Python shebangs in all files
find . -type f -name "*" -exec sed -i 's|^#![  ]*/usr/bin/env.*python.*$|#!/usr/bin/python3|' {} 2>/dev/null ';'

# test scripts
cp %{SOURCE1} scripts/
cp %{SOURCE2} scripts/
chmod +x scripts/*.sh
sed -i 's/ python / python3 /' scripts/run_python_examples.sh
sed -i 's|build/|./|' scripts/run_cpp_examples.sh

# builddir for serial
mkdir build-serial

%if %{with mpich}
    mkdir build-mpich
%endif

%if %{with openmpi}
    mkdir build-openmpi
%endif

%build
# Best to use && so that if anything in the chain fails, the build also fails
# straight away
%global do_cmake_config %{expand: \
echo
echo "*** BUILDING arbor-%{version}$MPI_COMPILE_TYPE ***"
echo
pushd build$MPI_COMPILE_TYPE  &&
    cmake \\\
        -DCMAKE_C_FLAGS_RELEASE:STRING="-DNDEBUG" \\\
        -DCMAKE_CXX_FLAGS_RELEASE:STRING="-DNDEBUG" \\\
        -DCMAKE_Fortran_FLAGS_RELEASE:STRING="-DNDEBUG" \\\
        -DCMAKE_VERBOSE_MAKEFILE:BOOL=ON \\\
        -DCMAKE_INSTALL_PREFIX:PATH=$MPI_HOME \\\
        -DINCLUDE_INSTALL_DIR:PATH=$MPI_INCLUDE \\\
        -DLIB_INSTALL_DIR:PATH=$MPI_LIB \\\
        -DSYSCONF_INSTALL_DIR:PATH=%{_sysconfdir} \\\
        -DSHARE_INSTALL_PREFIX:PATH=%{_datadir} \\\
        -DCMAKE_SKIP_RPATH:BOOL=ON \\\
        -DCMAKE_BUILD_TYPE:STRING="release" \\\
        -DARB_USE_BUNDLED_LIBS:BOOL=OFF \\\
        -DARB_USE_BUNDLED_PYBIND11:BOOL=OFF \\\
%ifarch x86_64 i686 aarch64
        -DARB_VECTORIZE:BOOL=ON \\\
%else
        -DARB_VECTORIZE:BOOL=OFF \\\
%endif
        -DARB_WITH_MPI:BOOL=$MPI_YES \\\
        -DARB_WITH_GPU:BOOL=OFF \\\
%ifnarch s390x s390
        -DARB_ARCH:STRING="native" \\\
%else
        -DARB_ARCH:STRING="none" \\\
%endif
        -DCMAKE_INSTALL_LIBDIR=%{_lib} \\\
        -DARB_WITH_PYTHON:BOOL=ON \\\
        -DARB_PYTHON_LIB_PATH:STRING=$MPI_PYTHON3_SITEARCH \\\
%if "%{_lib}" == "lib64"
        -DLIB_SUFFIX=64 .. &&
%else
        -DLIB_SUFFIX=""  .. &&
%endif
popd || exit -1;

# Upstream only supports static libraries
# https://github.com/arbor-sim/arbor/issues/916
# -DBUILD_SHARED_LIBS:BOOL=ON \\\
# Missing BRs
# -DARB_BUILD_VALIDATION_DATA:BOOL=ON \\\
}

%global do_make_build %{expand: \
    %make_build -C build$MPI_COMPILE_TYPE || exit -1
    %make_build -C build$MPI_COMPILE_TYPE examples || exit -1
%if %{with tests}
    %make_build -C build$MPI_COMPILE_TYPE tests || exit -1
%endif
}

# Build serial version, dummy arguments
%global __cc gcc
%global __cxx g++
%set_build_flags
export CC=gcc
export CXX=g++
export MPI_SUFFIX=""
export MPI_HOME=%{_prefix}
export MPI_INCLUDE=%{_includedir}
export MPI_LIB=%{_libdir}
export MPI_YES=OFF
export MPI_COMPILE_TYPE="-serial"
export MPI_PYTHON3_SITEARCH=%{python3_sitearch}
%{do_cmake_config}
%{do_make_build}

# Manually make docs in the serial build
sphinx-build-%{python3_version} -b html doc html
# Remove uneeded dotfiles
rm -rfv html/{.buildinfo,.doctrees}


# Build mpich version
%if %{with mpich}
%{_mpich_load}
%global __cc mpicc
%global __cxx mpicxx
%set_build_flags
export CC=mpicc
export CXX=mpicxx
export FC=mpif90
export F77=mpif77
export MPI_YES=ON
export MPI_COMPILE_TYPE="-mpich"
%{do_cmake_config}
%{do_make_build}

%{_mpich_unload}
%endif

# Build OpenMPI version
%if %{with openmpi}
%{_openmpi_load}
%global __cc mpicc
%global __cxx mpicxx
%set_build_flags
export CC=mpicc
export CXX=mpicxx
export FC=mpif90
export F77=mpif77
export MPI_YES=ON
# Python 3
export MPI_COMPILE_TYPE="-openmpi"
%{do_cmake_config}
%{do_make_build}

%{_openmpi_unload}
%endif

%install
# Install everything
%global do_install %{expand: \
echo
echo "*** INSTALLING arbor-%{version}$MPI_COMPILE_TYPE ***"
echo
    %make_install -C build$MPI_COMPILE_TYPE || exit -1
}

# install serial version
export MPI_SUFFIX=""
export MPI_HOME=%{_prefix}
export MPI_BIN=%{_bindir}
export MPI_YES=OFF
export MPI_COMPILE_TYPE="-serial"
%{do_install}


# Install MPICH version
%if %{with mpich}
%{_mpich_load}
export MPI_COMPILE_TYPE="-mpich"
%{do_install}

# Place in correct mpi libdir
%if "%{_lib}" == "lib64"
    mv -v $RPM_BUILD_ROOT/%{_libdir}/mpich/lib64 $RPM_BUILD_ROOT/$MPI_LIB/
%endif

pushd $RPM_BUILD_ROOT/$MPI_BIN
    mv -v modcc{,$MPI_SUFFIX} -v
popd
%{_mpich_unload}
%endif

# Install OpenMPI version
%if %{with openmpi}
%{_openmpi_load}
export MPI_COMPILE_TYPE="-openmpi"
%{do_install}

# Correct location
%if "%{_lib}" == "lib64"
    mv -v $RPM_BUILD_ROOT/%{_libdir}/openmpi/lib64 $RPM_BUILD_ROOT/$MPI_LIB/
%endif

pushd $RPM_BUILD_ROOT/$MPI_BIN
    mv -v modcc{,$MPI_SUFFIX} -v
popd
%{_openmpi_unload}
%endif


# https://github.com/arbor-sim/arbor/blob/master/.github/workflows/basic.yml
%if %{with tests}
%check
# General tests
export PYTHONPATH="%{buildroot}%{python3_sitearch}:%{buildroot}%{python3_sitelib}"

# these tests segfault, filter out
pushd build-serial
    %{python3} ../python/test/unit/runner.py -v2
    ./bin/unit
    ./bin/unit-modcc
    cp ../scripts/run_cpp_examples.sh .
    ./run_cpp_examples.sh
popd
./scripts/run_python_examples.sh

# MPICH
%if %{with mpich}
%{_mpich_load}
export PYTHONPATH="%{buildroot}/$MPI_PYTHON3_SITEARCH"
pushd build-mpich
    #./bin/unit --gtest_filter=-*mc_event_delivery*:*fvm_lowered*:*mc_cell_group*
    mpirun -n %{_smp_build_ncpus} %{python3} ../python/test/unit_distributed/runner.py -v2
    mpirun -n %{_smp_build_ncpus} ./bin/unit-mpi
    cp ../scripts/run_cpp_examples.sh .
    ./run_cpp_examples.sh "mpirun -n %{_smp_build_ncpus}"
popd
./scripts/run_python_examples.sh  "mpirun -n %{_smp_build_ncpus}"
%{_mpich_unload}
%endif

# OpenMPI
%if %{with openmpi}
%{_openmpi_load}
export PYTHONPATH="%{buildroot}/$MPI_PYTHON3_SITEARCH"
pushd build-openmpi
    mpirun -n %{_smp_build_ncpus} %{python3} ../python/test/unit_distributed/runner.py -v2
    mpirun -n %{_smp_build_ncpus} ./bin/unit-mpi
    cp ../scripts/run_cpp_examples.sh .
    ./run_cpp_examples.sh "mpirun -n %{_smp_build_ncpus}"
popd
./scripts/run_python_examples.sh  "mpirun -n %{_smp_build_ncpus}"
%{_openmpi_unload}
%endif
%endif


%files
%license LICENSE ext/tinyopt/LICENSE-tinyopt
%doc README.md
%{_bindir}/modcc
%{python3_sitearch}/arbor


%files devel
%{_includedir}/arborio
%{_includedir}/arbor
%{_includedir}/arborenv
%{_libdir}/cmake/arbor
%{_libdir}/libarbor.a
%{_libdir}/libarborio.a
%{_libdir}/libarborenv.a

%files doc
%license LICENSE
%doc html


%if %{with mpich}
%files mpich
%doc README.md
%license LICENSE ext/tinyopt/LICENSE-tinyopt
%{_libdir}/mpich/bin/modcc_mpich
%{python3_sitearch}/mpich/arbor

%files mpich-devel
%{_libdir}/mpich/include/arbor
%{_libdir}/mpich/include/arborio
%{_libdir}/mpich/include/arborenv
%{_libdir}/mpich/lib/cmake/arbor
%{_libdir}/mpich/lib/libarbor.a
%{_libdir}/mpich/lib/libarborio.a
%{_libdir}/mpich/lib/libarborenv.a
%endif

%if %{with openmpi}
%files openmpi
%doc README.md
%license LICENSE ext/tinyopt/LICENSE-tinyopt
%{_libdir}/openmpi/bin/modcc_openmpi
%{python3_sitearch}/openmpi/arbor

%files openmpi-devel
%{_libdir}/openmpi/include/arbor
%{_libdir}/openmpi/include/arborio
%{_libdir}/openmpi/include/arborenv
%{_libdir}/openmpi/lib/cmake/arbor
%{_libdir}/openmpi/lib/libarbor.a
%{_libdir}/openmpi/lib/libarborio.a
%{_libdir}/openmpi/lib/libarborenv.a
%endif

%changelog
* Wed Jul 20 2022 Fedora Release Engineering <releng@fedoraproject.org> 0.5.2-8
- Rebuilt for https://fedoraproject.org/wiki/Fedora_37_Mass_Rebuild

* Tue Jul 05 2022 Mamoru TASAKA <mtasaka@fedoraproject.org> 0.5.2-7
- Fix FTBFS on F37 - Workaround for %%%%__cc macro expansion issue (bug
  2103479) - backport upstream fix for test failure wrt move instruction
  omission on ctor with recent compiler - backport upstream fix for pandas
  error with duplicate indices

* Wed Jun 15 2022 Python Maint <python-maint@redhat.com> 0.5.2-6
- Rebuilt for Python 3.11

* Wed Jan 19 2022 Fedora Release Engineering <releng@fedoraproject.org> 0.5.2-5
- Rebuilt for https://fedoraproject.org/wiki/Fedora_36_Mass_Rebuild

* Sat Sep 25 2021 Ankur Sinha (Ankur Sinha Gmail) <sanjay.ankur@gmail.com> 0.5.2-4
- chore: remove second unneeded patch

* Sat Sep 25 2021 Ankur Sinha (Ankur Sinha Gmail) <sanjay.ankur@gmail.com> 0.5.2-3
- feat: correct flags for s390 builds

* Sat Sep 25 2021 Ankur Sinha (Ankur Sinha Gmail) <sanjay.ankur@gmail.com> 0.5.2-2
- chore: remove unused patch

* Sat Sep 25 2021 Ankur Sinha (Ankur Sinha Gmail) <sanjay.ankur@gmail.com> 0.5.2-1
- feat: update to 0.5.2

* Wed Jul 21 2021 Fedora Release Engineering <releng@fedoraproject.org> 0.3-13
- Rebuilt for https://fedoraproject.org/wiki/Fedora_35_Mass_Rebuild

* Wed Jul 21 2021 Fedora Release Engineering <releng@fedoraproject.org> 0.3-12
- Rebuilt for https://fedoraproject.org/wiki/Fedora_35_Mass_Rebuild

* Fri Jun 04 2021 Python Maint <python-maint@redhat.com> 0.3-11
- Rebuilt for Python 3.10

* Tue Jan 26 2021 Fedora Release Engineering <releng@fedoraproject.org> 0.3-10
- Rebuilt for https://fedoraproject.org/wiki/Fedora_34_Mass_Rebuild

* Wed Dec 16 2020 Tom Stellard <tstellar@redhat.com> 0.3-9
- Add BuildRequires: make

* Mon Oct 05 2020 Ankur Sinha (Ankur Sinha Gmail) <sanjay.ankur@gmail.com> 0.3-8
- Explicitly require setuptools

* Fri Aug 28 2020 Jeff Law <law@redhat.com> 0.3-7
- Re-enable LTO Do not force -march=native on armv7hl so that build
  architecture does not bleed into the binaries.  This should probably be
  done on all targets

* Sat Aug 01 2020 Fedora Release Engineering <releng@fedoraproject.org> 0.3-6
- Second attempt - Rebuilt for
  https://fedoraproject.org/wiki/Fedora_33_Mass_Rebuild

* Mon Jul 27 2020 Jeff Law <law@redhat.com> 0.3-5
- Disable LTO for armv7hl build

* Mon Jul 27 2020 Fedora Release Engineering <releng@fedoraproject.org> 0.3-4
- Rebuilt for https://fedoraproject.org/wiki/Fedora_33_Mass_Rebuild

* Sun Jun 28 2020 Ankur Sinha (Ankur Sinha Gmail) <sanjay.ankur@gmail.com> 0.3-3
- Update compiler and compiler flags

* Mon Jun 08 2020 sagitter <sagitter@fedoraproject.org> 0.3-2
- Move Provides lines to runtime packages

* Mon Jun 08 2020 sagitter <sagitter@fedoraproject.org> 0.3-1
- Release 0.3

* Tue May 26 2020 Miro Hrončok <miro@hroncok.cz> 0.2.2-3
- Rebuilt for Python 3.9

* Thu Apr 02 2020 Björn Esser <besser82@fedoraproject.org> 0.2.2-2
- Bump release

* Thu Apr 02 2020 Björn Esser <besser82@fedoraproject.org> 0.2.2-1
- Fix string quoting for rpm >= 4.16

* Wed Feb 26 2020 Ankur Sinha (Ankur Sinha Gmail) <sanjay.ankur@gmail.com>
- Update to new commit: fix 32bit builds

* Sun Feb 23 2020 Ankur Sinha (Ankur Sinha Gmail) <sanjay.ankur@gmail.com>
- Fix build using snapshot

* Tue Jan 28 2020 Fedora Release Engineering <releng@fedoraproject.org>
- Rebuilt for https://fedoraproject.org/wiki/Fedora_32_Mass_Rebuild

* Tue Dec 10 2019 Ankur Sinha (Ankur Sinha Gmail) <sanjay.ankur@gmail.com>
- Initial import
